use strict;
use warnings;

# CELL_SITE_UNIT_TESTING
#        used only for App::CELL unit tests
set('CELL_SITE_UNIT_TESTING', [ 'Om mane padme hum' ] );

# CELL_SITE_LOAD_OK
#        used by App::CELL::Load::init for sanity check
set('CELL_SITE_LOAD_OK', 'YES');

1;
